/* tslint:disable */
/* eslint-disable */
export const memory: WebAssembly.Memory;
export function get_cid(a: number, b: number, c: number): void;
export function __wbindgen_malloc(a: number): number;
export function __wbindgen_free(a: number, b: number): void;
